@@ -18,13 +18,14 @@ This contains functions to manage the firmware configuration of Lenovo servers
 """
 
 import ast
+import base64
 import random
 import struct
 
-import pyghmi.exceptions as pygexc
-from pyghmi.ipmi.oem.lenovo import EfiDecompressor
 import six
 
+import pyghmi.exceptions as pygexc
+
 try:
     import EfiCompressor
     from lxml import etree
@@ -36,16 +37,24 @@ IMM_NETFN = 0x2e
 IMM_COMMAND = 0x90
 LENOVO_ENTERPRISE = [0x4d, 0x4f, 0x00]
 
-OPEN_RO_COMMAND = [0x01, 0x05, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10]
+OPEN_RO_COMMAND = [0x01, 0x05, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40]
 OPEN_WO_COMMAND = [0x01, 0x03, 0x01]
 READ_COMMAND = [0x02]
 WRITE_COMMAND = [0x03]
 CLOSE_COMMAND = [0x05]
 SIZE_COMMAND = [0x06]
 
+class Unsupported(Exception):
+    pass
+
+def fromstring(inputdata):
+    if b'!entity' in inputdata.lower():
+        raise Exception('Unsupported XML')
+    return etree.fromstring(inputdata)
+
 
 def run_command_with_retry(connection, data):
-    tries = 10
+    tries = 240
     while tries:
         tries -= 1
         try:
@@ -72,8 +81,8 @@ class _ExpEngine(object):
         for optkey in self.cfg:
             opt = self.cfg[optkey]
             lid = opt['lenovo_id'].replace('-', '_')
-            if (lid == category and
-                    opt['lenovo_setting'] == setting):
+            if (lid == category
+                    and opt['lenovo_setting'] == setting):
                 self.relatedsettings.add(optkey)
                 return opt['lenovo_value']
         return None
@@ -126,11 +135,15 @@ def _eval_conditional(expression, cfg, setting):
 
 
 class LenovoFirmwareConfig(object):
-    def __init__(self, ipmicmd):
+    def __init__(self, xc, useipmi=True):
         if not etree:
             raise Exception("python-lxml and python-eficompressor required "
                             "for this function")
-        self.connection = ipmicmd
+        if useipmi:
+            self.connection = xc.ipmicmd
+        else:
+            self.connection = None
+        self.xc = xc
 
     def imm_size(self, filename):
         data = bytearray()
@@ -159,7 +172,7 @@ class LenovoFirmwareConfig(object):
             data.extend(OPEN_WO_COMMAND)
             hex_size = struct.pack("<I", size)
             data.extend(bytearray(hex_size[:4]))
-            data.extend([0x01, 0x10])
+            data.extend([0x01, 0x40])
         if not isinstance(filename, bytes):
             filename = filename.encode('utf-8')
         data.extend(filename)
@@ -186,7 +199,11 @@ class LenovoFirmwareConfig(object):
 
         hex_filehandle = struct.pack("<I", filehandle)
         data.extend(bytearray(hex_filehandle[:4]))
-        run_command_with_retry(self.connection, data=data)
+        try:
+            run_command_with_retry(self.connection, data=data)
+        except pygexc.IpmiException as e:
+            if e.ipmicode != 203:
+                raise
 
     def imm_write(self, filehandle, size, inputdata):
         blocksize = 0xc8
@@ -210,6 +227,7 @@ class LenovoFirmwareConfig(object):
             remaining -= blocksize
             offset += blocksize
             run_command_with_retry(self.connection, data=data)
+            self.connection.ipmi_session.pause(0)
 
     def imm_read(self, filehandle, size):
         blocksize = 0xc8
@@ -232,10 +250,9 @@ class LenovoFirmwareConfig(object):
             data.extend(hex_blocksize[:2])
             remaining -= blocksize
             offset += blocksize
-
             response = run_command_with_retry(self.connection, data=data)
             output += response['data'][5:]
-
+            self.connection.ipmi_session.pause(0)
         return output
 
     def factory_reset(self):
@@ -254,18 +271,28 @@ class LenovoFirmwareConfig(object):
             cfgfilename = "config"
         options = {}
         data = None
-        for _ in range(0, 30):
-            filehandle = self.imm_open(cfgfilename)
-            size = self.imm_size(cfgfilename)
-            data = self.imm_read(filehandle, size)
-            self.imm_close(filehandle)
-            data = EfiDecompressor.decompress(data)
-            if len(data) != 0:
-                break
-            self.connection.ipmi_session.pause(2)
+        rsp = self.xc.grab_redfish_response_with_status(
+            '/redfish/v1/Systems/1/Actions/Oem/LenovoComputerSystem.DSReadFile',
+            {'Action': 'DSReadFile', 'FileName': cfgfilename})
+        if rsp[1] == 200:
+            data = rsp[0]['Content']
+            data = base64.b64decode(data)
+            data = EfiCompressor.FrameworkDecompress(data, len(data))
+        else:
+            if self.connection is None:
+                raise Unsupported('Not Supported')
+            for _ in range(0, 30):
+                filehandle = self.imm_open(cfgfilename)
+                size = self.imm_size(cfgfilename)
+                data = self.imm_read(filehandle, size)
+                self.imm_close(filehandle)
+                data = EfiCompressor.FrameworkDecompress(data, len(data))
+                if len(data) != 0:
+                    break
+                self.connection.ipmi_session.pause(2)
         if not data:
             raise Exception("BMC failed to return configuration information")
-        xml = etree.fromstring(data)
+        xml = fromstring(data)
         sortid = 0
         for config in xml.iter("config"):
             lenovo_id = config.get("ID")
@@ -295,6 +322,10 @@ class LenovoFirmwareConfig(object):
                     readonly = setting.get('gray-if')
                     if readonly:
                         readonly = _convert_syntax(readonly)
+                    else:
+                        access = setting.get('access')
+                        if access == 'readonly':
+                            readonly = 'true'
                     possible = []
                     current = None
                     default = None
@@ -322,8 +353,8 @@ class LenovoFirmwareConfig(object):
                         default = onedata.get('default', None)
                         if default == '':
                             default = None
-                    if (setting.find('cmd_data') is not None or
-                            setting.find('boolean_data') is not None):
+                    if (setting.find('cmd_data') is not None
+                            or setting.find('boolean_data') is not None):
                         protect = True  # Hide currently unsupported settings
                     ldata = setting.find("list_data")
                     extraorder = False
@@ -462,7 +493,7 @@ class LenovoFirmwareConfig(object):
 
         return options
 
-    def set_fw_options(self, options):
+    def set_fw_options(self, options, checkonly=False):
         changes = False
         random.seed()
         ident = 'ASU-%x-%x-%x-0' % (random.getrandbits(48),
@@ -475,10 +506,6 @@ class LenovoFirmwareConfig(object):
         for option in options.keys():
             if options[option]['new_value'] is None:
                 continue
-            if options[option]['current'] == options[option]['new_value']:
-                continue
-            if options[option]['pending'] == options[option]['new_value']:
-                continue
             if options[option]['readonly']:
                 errstr = '{0} is read only'.format(option)
                 if options[option]['readonly_why']:
@@ -486,6 +513,10 @@ class LenovoFirmwareConfig(object):
                         ','.join(sorted(options[option]['readonly_why'])))
                     errstr += ea
                 raise pygexc.InvalidParameterValue(errstr)
+            if options[option]['current'] == options[option]['new_value']:
+                continue
+            if options[option]['pending'] == options[option]['new_value']:
+                continue
             if isinstance(options[option]['new_value'], six.string_types):
                 # Coerce a simple string parameter to the expected list format
                 options[option]['new_value'] = [options[option]['new_value']]
@@ -521,19 +552,41 @@ class LenovoFirmwareConfig(object):
                 if is_list:
                     count += 1
                     instance = etree.Element(
-                        'instance', ID=options[option]['lenovo_instance'],
+                        'instance', ID=str(options[option]['lenovo_instance']),
                         order=str(count))
                 else:
                     instance = etree.Element(
-                        'instance', ID=options[option]['lenovo_instance'])
+                        'instance', ID=str(options[option]['lenovo_instance']))
                 choice.append(instance)
 
         if not changes:
-            return
+            return False
+        if checkonly:
+            return True
 
         xml = etree.tostring(configurations)
         data = EfiCompressor.FrameworkCompress(xml, len(xml))
+        bdata = base64.b64encode(data).decode('utf8')
+        rsp = self.xc.grab_redfish_response_with_status(
+                '/redfish/v1/Managers/1')
+        if rsp[1] == 200:
+            if 'purley' not in rsp[0].get('Oem', {}).get('Lenovo', {}).get(
+                    'release_name', 'purley'):
+                rsp = self.xc.grab_redfish_response_with_status(
+                    '/redfish/v1/Systems/1/Actions/Oem/'
+                    'LenovoComputerSystem.DSWriteFile',
+                    {'Action': 'DSWriteFile', 'Resize': len(data),
+                     'FileName': 'asu_update.efi', 'Content': bdata})
+                if rsp[1] == 204:
+                    return True
+        if self.connection is None:
+            raise Unsupported('Not Supported')
         filehandle = self.imm_open("asu_update.efi", write=True,
-                                   size=len(data))
+                                size=len(data))
         self.imm_write(filehandle, len(data), data)
+        stubread = len(data)
+        if stubread > 8:
+            stubread = 8
+        self.imm_read(filehandle, stubread)
         self.imm_close(filehandle)
+        return True
