@@ -383,6 +383,10 @@ class OEMHandler(generic.OEMHandler):
                 raise pygexc.InvalidParameterValue("Index must be 0 or 1")
             return self.tsmahandler.set_ntp_server(server, index)
         return None
+    
+    def user_delete(self, uid, channel):
+        if self.has_xcc:
+            return self.immhandler.user_delete(uid)
 
     def set_user_access(self, uid, channel, callback, link_auth, ipmi_msg,
                         privilege_level):
@@ -1233,6 +1237,8 @@ class OEMHandler(generic.OEMHandler):
             return self.immhandler.clear_bmc_configuration()
         elif self.is_fpc:
             return self.smmhandler.clear_bmc_configuration()
+        elif self.has_tsma:
+            return self.tsmahandler.clear_bmc_configuration()
         return super(OEMHandler, self).clear_system_configuration()
 
     def clear_system_configuration(self):
@@ -1310,6 +1316,15 @@ class OEMHandler(generic.OEMHandler):
             return True
         return False
 
+    def get_user_privilege_level(self, uid):
+        if self.has_xcc:
+            return self.immhandler.get_user_privilege_level(uid)
+        return None
+
+    def set_user_access(self, uid, channel, callback, link_auth, ipmi_msg, privilege_level):
+        if self.has_xcc:
+            self.immhandler.set_user_access(uid, privilege_level)
+
     def process_zero_fru(self, zerofru):
         if (self.oemid['manufacturer_id'] == 19046
                 and self.oemid['product_id'] == 13616):
