@@ -26,6 +26,7 @@ import six
 
 import pyghmi.exceptions as pygexc
 import pyghmi.ipmi.private.util as util
+import pyghmi.ipmi.oem.lenovo.config as config
 import pyghmi.media as media
 import pyghmi.redfish.oem.generic as generic
 import pyghmi.storage as storage
@@ -43,6 +44,22 @@ funtypes = {
     12: 'Fabric Controller',
 }
 
+tls_ver = {
+    1: 'TLS1.1',
+    2: 'TLS1.2',
+    3: 'TLS1.3'
+}
+
+sec_mode = {
+    0: ('Compatible',
+        'Maximum compatibility of cipher suites, without NIST compliance'),
+    1: ('NIST',
+        'Maximum compatibility within the constraints of being NIST '
+        'compliant with perfect forward secrecy'),
+    2: ('High', 'Support only modern and strong ciphers, NIST '
+                'compliant with perfect forward security'),
+}
+
 
 def naturalize_string(key):
     """Analyzes string in a human way to enable natural sort
@@ -95,6 +112,134 @@ class OEMHandler(generic.OEMHandler):
         self.weblogging = False
         self.updating = False
         self.datacache = {}
+        self.fwc = None
+        self.fwo = None
+
+    def get_system_configuration(self, hideadvanced=True, fishclient=None,
+                                 fetchimm=False):
+        if not self.fwc:
+            self.fwc = config.LenovoFirmwareConfig(self, useipmi=False)
+        try:
+            self.fwo = self.fwc.get_fw_options(fetchimm=fetchimm)
+        except config.Unsupported:
+            return super(OEMHandler, self).get_system_configuration(
+                hideadvanced, fishclient)
+        except Exception:
+            raise Exception('%s failed to retrieve UEFI configuration'
+                            % self.bmcname)
+        self.fwovintage = util._monotonic_time()
+        retcfg = {}
+        for opt in self.fwo:
+            if 'MegaRAIDConfigurationTool' in opt:
+                # Suppress the Avago configuration to be consistent with
+                # other tools.
+                continue
+            if (hideadvanced and self.fwo[opt]['lenovo_protect']
+                    or self.fwo[opt]['hidden']):
+                # Do not enumerate hidden settings
+                continue
+            retcfg[opt] = {}
+            retcfg[opt]['value'] = self.fwo[opt]['current']
+            retcfg[opt]['default'] = self.fwo[opt]['default']
+            retcfg[opt]['help'] = self.fwo[opt]['help']
+            retcfg[opt]['possible'] = self.fwo[opt]['possible']
+            retcfg[opt]['sortid'] = self.fwo[opt]['sortid']
+        return retcfg
+
+    def set_system_configuration(self, changeset, fishclient):
+        if not self.fwc:
+            self.fwc = config.LenovoFirmwareConfig(self, useipmi=False)
+        fetchimm = False
+        if not self.fwo or util._monotonic_time() - self.fwovintage > 30:
+            try:
+                self.fwo = self.fwc.get_fw_options(fetchimm=fetchimm)
+            except config.Unsupported:
+                return super(OEMHandler, self).set_system_configuration(
+                    changeset, fishclient)
+            self.fwovintage = util._monotonic_time()
+        for key in list(changeset):
+            if key not in self.fwo:
+                found = False
+                for rkey in self.fwo:
+                    if fnmatch.fnmatch(rkey.lower(), key.lower()):
+                        changeset[rkey] = changeset[key]
+                        found = True
+                    elif self.fwo[rkey].get('alias', None) != rkey:
+                        calias = self.fwo[rkey]['alias']
+                        if fnmatch.fnmatch(calias.lower(), key.lower()):
+                            changeset[rkey] = changeset[key]
+                            found = True
+                if not found and not fetchimm:
+                    fetchimm = True
+                    self.fwo = self.fwc.get_fw_options(fetchimm=fetchimm)
+                    if key in self.fwo:
+                        continue
+                    else:
+                        found = False
+                        for rkey in self.fwo:
+                            if fnmatch.fnmatch(rkey.lower(), key.lower()):
+                                changeset[rkey] = changeset[key]
+                                found = True
+                            elif self.fwo[rkey].get('alias', None) != rkey:
+                                calias = self.fwo[rkey]['alias']
+                                if fnmatch.fnmatch(
+                                        calias.lower(), key.lower()):
+                                    changeset[rkey] = changeset[key]
+                                    found = True
+                if found:
+                    del changeset[key]
+                else:
+                    raise pygexc.InvalidParameterValue(
+                        '{0} not a known setting'.format(key))
+        self.merge_changeset(changeset)
+        if changeset:
+            try:
+                self.fwc.set_fw_options(self.fwo)
+            finally:
+                self.fwo = None
+                self.fwovintage = 0
+
+    def merge_changeset(self, changeset):
+        for key in changeset:
+            if isinstance(changeset[key], six.string_types):
+                changeset[key] = {'value': changeset[key]}
+            newvalue = changeset[key]['value']
+            if self.fwo[key]['is_list'] and not isinstance(newvalue, list):
+                if '=' in newvalue:
+                    # ASU set a precedent of = delimited settings
+                    # for now, honor that delimiter as well
+                    newvalues = newvalue.split('=')
+                else:
+                    newvalues = newvalue.split(',')
+            else:
+                newvalues = [newvalue]
+            newnewvalues = []
+            for newvalue in newvalues:
+                newv = re.sub(r'\s+', ' ', newvalue)
+                if (self.fwo[key]['possible']
+                        and newvalue not in self.fwo[key]['possible']):
+                    candlist = []
+                    for candidate in self.fwo[key]['possible']:
+                        candid = re.sub(r'\s+', ' ', candidate)
+                        if newv.lower().startswith(candid.lower()):
+                            newvalue = candidate
+                            break
+                        if candid.lower().startswith(newv.lower()):
+                            candlist.append(candidate)
+                    else:
+                        if len(candlist) == 1:
+                            newvalue = candlist[0]
+                        else:
+                            raise pygexc.InvalidParameterValue(
+                                '{0} is not a valid value for {1} '
+                                '({2})'.format(
+                                    newvalue, key,
+                                    ','.join(self.fwo[key]['possible'])))
+                newnewvalues.append(newvalue)
+            if len(newnewvalues) == 1:
+                self.fwo[key]['new_value'] = newnewvalues[0]
+            else:
+                self.fwo[key]['new_value'] = newnewvalues
 
     def reseat_bay(self, bay):
         if bay != -1:
@@ -138,6 +283,26 @@ class OEMHandler(generic.OEMHandler):
             settings['presence_assert'] = {
                 'value': 'Enable' if asserted else 'Disable'
             }
+        secparms = self.wc.grab_json_response('/api/providers/imm_tls_mode')
+        if secparms and secparms.get('return', 1) == 0:
+            tlsmode = secparms.get('tls_ver', -1)
+            if tlsmode in tls_ver:
+                settings['minimum_tls_version'] = {
+                    'value': tls_ver[tlsmode],
+                    'help': 'The minimum TLS level allowed by the XCC when '
+                            'communicating',
+                    'possible': [tls_ver[x] for x in tls_ver],
+                }
+            secmode = secparms.get('sec_mode', -1)
+            if secmode in sec_mode:
+                settings['cryptography_mode'] = {
+                    'value': sec_mode[secmode][0],
+                    'help': 'Select cryptography mode, compatible allows all '
+                            'supported ciphers, NIST restricts to NIST '
+                            'compliant ciphers, and High further restricts '
+                            'to a modern subset of NIST compliance',
+                    'possible': [sec_mode[x][0] for x in sec_mode],
+                }
         usbparms = self.wc.grab_json_response('/api/dataset/imm_usb')
         if usbparms:
             usbparms = usbparms.get('items', [{}])[0]
@@ -177,6 +342,7 @@ class OEMHandler(generic.OEMHandler):
     def set_bmc_configuration(self, changeset):
         ruleset = {}
         usbsettings = {}
+        secparms = {}
         for key in changeset:
             if isinstance(changeset[key], six.string_types):
                 changeset[key] = {'value': changeset[key]}
@@ -203,14 +369,57 @@ class OEMHandler(generic.OEMHandler):
                     'usb_ethernet', 'usb_ethernet_port_forwarding',
                     'usb_forwarded_ports'):
                 usbsettings[key] = changeset[key]['value']
+            elif key.lower() in ('cryptography_mode', 'minimum_tls_version'):
+                secparms[key] = changeset[key]['value']
             else:
                 raise pygexc.InvalidParameterValue(
                     '{0} not a known setting'.format(key))
         if usbsettings:
             self.apply_usb_configuration(usbsettings)
+        if secparms:
+            self.apply_sec_configuration(secparms)
         if ruleset:
             self.wc.grab_json_response('/api/dataset', ruleset)
 
+    def apply_sec_configuration(self, secparms):
+        secmode = None
+        tlsver = None
+        if 'cryptography_mode' in secparms:
+            secmode = secparms['cryptography_mode'].lower()
+            for sm in sec_mode:
+                if sec_mode[sm][0].lower().startswith(secmode):
+                    secmode = sm
+                    break
+            else:
+                raise pygexc.InvalidParameterValue(
+                    '"{0}" is not a recognized cryptography mode'.format(
+                        secparms['cryptography_mode']
+                    )
+                )
+        if 'minimum_tls_version' in secparms:
+            tlsver = secparms['minimum_tls_version'].lower()
+            for tv in tls_ver:
+                if tls_ver[tv].lower() == tlsver:
+                    tlsver = tv
+                    break
+            else:
+                raise pygexc.InvalidParameterValue(
+                    '"{0}" is not a recognized TLS version'.format(
+                        secparms['minimum_tls_version'])
+                )
+        if len(secparms) < 2:
+            currsecparms = self.wc.grab_json_response(
+                '/api/providers/imm_tls_mode')
+            del currsecparms['return']
+        else:
+            currsecparms = {}
+        if secmode is not None:
+            currsecparms['sec_mode'] = secmode
+        if tlsver is not None:
+            currsecparms['tls_ver'] = tlsver
+        self.wc.grab_json_response('/api/providers/imm_tls_mode',
+                                   currsecparms)
+
     def apply_usb_configuration(self, usbsettings):
         def numify(val):
             if 'enabled'.startswith(val.lower()):
@@ -762,6 +971,11 @@ class OEMHandler(generic.OEMHandler):
                 wc.set_header('X-XSRF-TOKEN', wc.cookies['_csrf_token'])
             return wc
 
+    def grab_redfish_response_with_status(self, url, body=None, method=None):
+        wc = self.webclient.dupe()
+        res = wc.grab_json_response_with_status(url, body, method=method)
+        return res
+
     def list_media(self, fishclient):
         rt = self.wc.grab_json_response('/api/providers/rp_vm_remote_getdisk')
         if 'items' in rt:
@@ -795,18 +1009,31 @@ class OEMHandler(generic.OEMHandler):
                 'XCC does not have required license for operation')
 
     def upload_media(self, filename, progress=None, data=None):
-        xid = random.randint(0, 1000000000)
+        wc = self.wc
         self._refresh_token()
-        uploadthread = webclient.FileUploader(
-            self.wc, '/upload?X-Progress-ID={0}'.format(xid), filename, data)
+        rsp, statu = wc.grab_json_response_with_status('/rdocupload')
+        newmode = False
+        if statu == 404:
+            xid = random.randint(0, 1000000000)
+            uploadthread = webclient.FileUploader(
+                wc, '/upload?X-Progress-ID={0}'.format(xid), filename, data)
+        else:
+            newmode = True
+            uploadthread = webclient.FileUploader(
+                wc, '/rdocupload', filename, data)
         uploadthread.start()
         while uploadthread.isAlive():
             uploadthread.join(3)
-            rsp = self.wc.grab_json_response(
-                '/upload/progress?X-Progress-ID={0}'.format(xid))
-            if progress and rsp['state'] == 'uploading':
-                progress({'phase': 'upload',
-                          'progress': 100.0 * rsp['received'] / rsp['size']})
+            if newmode:
+                if progress:
+                    progress({'phase': 'upload',
+                          'progress': 100 * wc.get_upload_progress()})
+            else:
+                rsp = self.wc.grab_json_response(
+                    '/upload/progress?X-Progress-ID={0}'.format(xid))
+                if progress and rsp['state'] == 'uploading':
+                    progress({'phase': 'upload',
+                            'progress': 100.0 * rsp['received'] / rsp['size']})
             self._refresh_token()
         rsp = json.loads(uploadthread.rsp)
         if progress:
@@ -1212,6 +1439,17 @@ class OEMHandler(generic.OEMHandler):
                     license_errors[rsp['return']])
         return self.get_licenses()
 
+    def user_delete(self, uid):
+        userinfo = self.wc.grab_json_response('/api/dataset/imm_users')
+        uidtonamemap = {}
+        for user in userinfo.get('items', [{'users': []}])[0].get('users', []):
+            uidtonamemap[user['users_user_id']] = user['users_user_name']
+        if uid in uidtonamemap:
+            deltarget = '{0},{1}'.format(uid, uidtonamemap[uid])
+            self.wc.grab_json_response('/api/function', {"USER_UserDelete": deltarget})
+            return True
+        return super(OEMHandler, self).user_delete(uid)
+
     def get_user_expiration(self, uid):
         userinfo = self.wc.grab_json_response('/api/dataset/imm_users')
         for user in userinfo['items'][0]['users']:
